#!/usr/bin/perl -T

# Copyright 2003 The Board of Trustees of 
# The Leland Stanford Junior University. All Rights Reserved.

# Communication to Jessica Shapiro, jessicas@stanford.edu

$PDBPATH = "/usr/db/pdb";
$DSSPPATH = "";
$SCOPPATH = "/usr/db/scop-1.63-pdb";
$SCOPDSSPPATH = "/usr/db/scop-1.63-dssp";

$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin";

#system commands used by this script:
$file = "/usr/bin/file";
$cat = "/bin/cat";
$ln = "/bin/ln";
$gunzip = "/bin/gunzip";
#$uncompress = "@UNCOMPRESS@";
$rm = "/bin/rm";
$mkdir = "/bin/mkdir";

open STDOUT, ">/dev/null";
open STDERR, ">/dev/null";


select STDOUT; $| = 1;
select STDERR; $| = 1;

$longpdbname = shift(@ARGV);
$newname = shift(@ARGV);


$newname = securityCheckFilename($newname);
if($newname !~ /\w/) {
  print "problem with newname ($newname)\n";
  exit(-1);
}

if (-e $newname) {
  system($rm, $newname);
}

if(length($longpdbname) == 7) { #if using scop id
  $pdbname = $longpdbname;
  $isScopID = 1;
  $pdbname = securityCheck($longpdbname,$isScopID); #flag indicates that this is a scop id
}else {
  $isScopID = 0;
  $pdbname = securityCheck($longpdbname,$isScopID);
}

$pdbname =~ tr/A-Z/a-z/;

if ((not defined $pdbname) || (not defined $newname)) {
  exit(0);
}

$myfile = findFile($pdbname, $isScopID);

if ($myfile ne "NOFILE") {
  
  open(OLD_STDOUT, ">&STDOUT");
  open(STDOUT,">/tmp/$$\.file");
  select(STDOUT); $| = 1;
  system($file,$myfile);
  close(STDOUT);
  open(STDOUT, ">&OLD_STDOUT");

  open(IN, "/tmp/$$\.file");
  $line = <IN>;
  close(IN);
  system("$rm","-f","/tmp/$$\.file");
  
  @ftype = split /\s+/, $line;

  
  if ($ftype[1] eq "gzip") {	
    open(OLD_STDOUT, ">&STDOUT");
    open(STDOUT,">$newname.gz") || die "can't open new stdout (/tmp/$newname.gz): $!\n";
    select(STDOUT); $| = 1;
    system($cat, $myfile);
    close(STDOUT);
    open(STDOUT, ">&OLD_STDOUT");
    system($gunzip, "$newname\.gz");
  }
  elsif (($ftype[1] eq "compressed") || ($ftype[1] eq "compress'd")) {	
    open(OLD_STDOUT, ">&STDOUT");
    open(STDOUT,">$newname.Z");
    select(STDOUT); $| = 1;
    system($cat, $myfile);
    close(STDOUT);
    open(STDOUT, ">&OLD_STDOUT");
    system($gunzip,"$newname.Z");
  }
  else {
    system($ln,"-s",$myfile,$newname);
  }
}

exit(0);

######################################################################

sub findFile() {
  my($filename, $isScopID) = @_;
  my($shortname);

  if(!$isScopID) { #pdb file
    ($two_letter) = $filename =~ /^[0-9](..)/;
    if($filename =~ /^(.*)-[A-Za-z0-9]$/) {
      $shortname = $1;
    }else {
      $shortname = $filename;
    }
    $mytemp = getFile("$PDBPATH/$two_letter/$shortname.pdb");

    if ($mytemp eq "NOFILE") {
      $mytemp = getFile("$PDBPATH/$shortname.pdb");
    }
    if ($mytemp eq "NOFILE") {
      $mytemp = getFile("$PDBPATH/pdb$shortname.ent");
    }
    if ($mytemp eq "NOFILE") {
      $mytemp = getFile("$PDBPATH/$two_letter/pdb$shortname.ent");
    }
    if ($mytemp eq "NOFILE") {
      $mytemp = getFile("$DSSPPATH/$shortname.dssp");
    }

  }else {
    $two_letter = substr($filename,2,2);

    $mytemp = getFile("$SCOPPATH/$two_letter/$filename.ent");

    if($mytemp eq "NOFILE") {
      $mytemp = getFile("$SCOPDSSPPATH/$two_letter/$filename.dssp");
    }

  }
  
  
    return $mytemp;
}


sub getFile() {
  
  my($mytemp) = @_;

#  print STDERR "looking for $mytemp\n";
  if (-e $mytemp) {
    return $mytemp;
  }
  
  $mytemp = "$_[0]".".gz";
  if (-e $mytemp) {
    return $mytemp;
  }
  
  $mytemp = "$_[0]".".Z";
  if (-e $mytemp) {
    return $mytemp;
  }
  
  return "NOFILE";
  
}
    
#checks user's pdb or scop id (can scop id start with anything other than d or g?)
sub securityCheck {
  my($inputname, $isScop) = @_;
  my($cleanname,$pdbid);
  

  $cleanname = "";
  if($isScop) {
    $inputname =~ /^([dg][0-9][A-Za-z0-9]{3}[\._A-Za-z0-9][_0-9])/;
    $cleanname = $1;

  }else {
    $inputname =~/^([0-9][A-Za-z0-9]{3})(-[A-Za-z0-9])?/;
    $cleanname = $1.$2; #$1 is the pdb id, and $2 is the chain, prefixed by '-'
  }
  return $cleanname; #need to do error checking -- could return empty string
}

sub securityCheckFilename {
  my($filename) = @_;
  #Make sure this is a full path that contains only alphanumeric characters and '/'
  if($filename =~ m|^((/var/www/html/FoldMiner2/results/[0-9]+/)?[\/A-Za-z\.0-9_]*)$|) {
    return $1;
  }else {
    return ""; #need to do error checking -- could return empty string
  }
}
