/* Copyright Amit P. Singh, Douglas L. Brutlag, Stanford University. */

#include <stdio.h> 
#include <stdlib.h> 
#include <string.h>
#include <math.h>
#include "defines.h" 

#ifdef DISPLAYON
#include <GL/gl.h> 
#endif

char usageErrorMessage[] = {
"\n\nlock usage:\n"
"\t-q  <pdbName1> \t\tquery pdb file name OR pdb code\n"
"\t-qc <chain1>   \t\tquery chain identifier (if present)\n"
"\t-t  <pdbName2> \t\ttarget pdb file name OR pdb code\n"
"\t-tc <chain2>   \t\ttarget chain identifier (if present)\n"
"\t-b             \t\trun in batch mode without graphical interface\n"
"\t-n             \t\tdo not compute core alignment\n"
"\t-o  <dirName>  \t\toutput directory name\n"
"\t-d  <threshold>\t\tthreshold value\n"
"\t-s             \t\thash using sequential vector pairs only\n"
"\t-h             \t\tprint this help \n"
"\n"
"Note:"
"\tThe query and target can be specified as either complete file names\n"
"\tor as pdb accession codes.  If the pdb codes are used, you can also\n"
"\tspecify a chain by concatenating the chain id to the pdb code as follows:\n"
"\tlock -q 1seb-A -t 1seb-B \n\n"
};

 
 
char threelettercode[22][4] = {  
  {"ALA"}, {"ARG"}, {"ASN"}, {"ASP"}, {"ASX"}, {"CYS"}, {"GLN"}, {"GLU"}, {"GLX"}, {"GLY"}, {"HIS"}, 
  {"ILE"}, {"LEU"}, {"LYS"}, {"MET"}, {"PHE"}, {"PRO"}, {"SER"}, {"THR"}, {"TRP"}, {"TYR"}, {"VAL"}
};

char onelettercode[23] = {"ARNDBCQEZGHILKMFPSTWYV"};


char username[2][300];
char dsspname[2][300]; /*holds names of corresponding dssp files, in case SSE info is needed*/
char filename[2][300];
char pdbname[2][13] = {{"query"}, {"target"}};
char chain[2]; 
int newfile[2];
FILE* filePtrs[2]; 
char outputDir[300];
float rotation[3][3];
float translation[3];
int runStep1 = 1;
extern int lock_residues;

float N[MAXLENGTH][3]; /*used to hold nitrogen coordinates*/
float C[MAXLENGTH][3]; /*used to hold carbon coordinates (carbonyl carbon*/

extern int useBetaCarbons;
extern int globalStartDisplay;
extern int globalComputeCore;
extern float distThreshold; 
extern float angleWeight;
extern float gapMultiplier;
extern float gapExtensionMultiplier;
extern float distanceWeight;
extern int mypid; 
extern int globalSearchDatabase;
extern FILE *globalDatabaseFile;
extern FILE *rasmolFilePtr;
extern char* lockdir;
extern char* scriptdir;
extern char* dsspprogram;
 
extern OBJ* ObjList[2]; 
extern void freeObj(OBJ*);
extern void setUseBetaCarbons(int);

#ifdef DISPLAYON
extern GLfloat colorTable[2][3];
extern void createObjList(OBJ*); 
#endif

OBJ* objectFromFile(int); 
OBJ* objectFromPDBFile(int); 
int getSSFromDSSPFile(OBJ*,int); 
OBJ* readFile(int); 
int noSSinPDBFile(OBJ*,int); 
void getCBAtoms(OBJ*, FILE*, int fnum);

void checkopts(int, char**); 
void usageError(); 
void extStr(char*, char*, int, int); 
char convert3to1Code(char*);
int aaToNumber(char);
void convert1to3Code(char, char*);
int checkIfList(FILE*);
void assignSSENumbers(OBJ* protein);
void assignSSArray(OBJ* myobject);
void findGlycineBetaCarbon(int index, OBJ* myobject, float *vec);

char* getFileName(int num) {
  if(num<=1) {
    return(filename[num]);
  }else
    return("-1");
}


void usageError() { 
 
  printf(usageErrorMessage); 
  exit(1); 
} 
 
int aaToNumber(char aa) {
  int number = -1;
  aa = (char)tolower((int)aa);

  switch (aa) {
  case 'a': number = 0;break;
  case 'c': number = 1; break;
  case 'd': number = 2; break;
  case 'e': number = 3; break;
  case 'f': number = 4; break;
  case 'g': number = 5; break;
  case 'h': number = 6; break;
  case 'i': number = 7; break;
  case 'k': number = 8; break;
  case 'l': number = 9; break;
  case 'm': number = 10; break;
  case 'n': number = 11; break;
  case 'p': number = 12; break;
  case 'q': number = 13; break;
  case 'r': number = 14; break;
  case 's': number = 15; break;
  case 't': number = 16; break;
  case 'v': number = 17; break;
  case 'w': number = 18; break;
  case 'y': number = 19; break;
  }
  return(number);
}

char convert3to1Code(char *code) {

  char ch;
  int i;

  ch = 'Z';

  for (i = 0; i < 22; i++) {
    if (strcmp(code, threelettercode[i]) == 0) {
      ch = onelettercode[i];
      break;
    }
  }
  return ch;
}

void convert1to3Code(char ch, char *retcode) {

  int i;

  strcpy(retcode, "NUL");

  for (i = 0; i < 22; i++) {
    if ((onelettercode[i] == ch) || (((char)((int)(onelettercode[i]) - 32)) == ch)) {
      strcpy(retcode, threelettercode[i]);
      break;
    }
  }
    
}

 
void checkopts(int argc, char **argv) { 
 
  int i;  
  int j,k;

  strcpy(username[0], "NONE");
  chain[0] = ' '; 
  strcpy(username[1], "NONE");
  chain[1] = ' '; 
  strcpy(outputDir, "NONE");
  strcpy(dsspname[0], "NONE");
  strcpy(dsspname[1], "NONE");



  for (i = 0; i < argc; i++) { 
 
    if (strcmp(argv[i], "-q") == 0) { 
 
      if (i < argc - 1) { 
	if (argv[i+1][0] != '-') {	   
	  strcpy(username[0], argv[i+1]); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
 
    else if (strcmp(argv[i], "-qc") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  chain[0] = argv[i+1][0]; 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
    else if(strcmp(argv[i], "-qdssp") == 0) {
      if(i < argc -1) {
	if(argv[i+1][0] != '-') {
	  strcpy(dsspname[0],argv[i+1]);
	  i++;
	}else {
	  usageError();
	}
      }else {
	usageError();
      }
    }
    else if (strcmp(argv[i], "-t") == 0) { 
 
      if (i < argc - 1) { 
	if (argv[i+1][0] != '-') {	   
	  strcpy(username[1], argv[i+1]); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
    else if(strcmp(argv[i], "-distance")==0) {
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  sscanf(argv[i+1], "%f", &distanceWeight); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
      
    }
    else if(strcmp(argv[i], "-angle")==0) {
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  sscanf(argv[i+1], "%f", &angleWeight); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
      
    }
    else if(strcmp(argv[i], "-open")==0) {
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') {
	  sscanf(argv[i+1], "%f", &gapMultiplier); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
      
    }
    else if(strcmp(argv[i], "-extend")==0) {
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  sscanf(argv[i+1], "%f", &gapExtensionMultiplier); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
      
    }
    else if (strcmp(argv[i], "-tc") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  chain[1] = argv[i+1][0]; 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
    else if(strcmp(argv[i], "-tdssp") == 0) {
      if(i < argc -1) {
	if(argv[i+1][0] != '-') {
	  strcpy(dsspname[1],argv[i+1]);
	  i++;
	}else {
	  usageError();
	}
      }else {
	usageError();
      }
    }
    else if (strcmp(argv[i], "-d") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  sscanf(argv[i+1], "%f", &distThreshold); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 

    else if (strcmp(argv[i], "-o") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  strcpy(outputDir, argv[i+1]); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 

    else if (strcmp(argv[i], "-b") == 0) { 

      globalStartDisplay = 0;
 
    } 

    else if (strcmp(argv[i], "-n") == 0) { 

      globalComputeCore = 0;
 
    } 
     else if(strcmp(argv[i], "-s") == 0) {
      printf("setting all pairs to zero\n");
      setAllPairs(0); /*use only sequential vector pairs in hashing*/

    }
     else if(strcmp(argv[i], "-transformation")==0) {
       printf("reading in transformation matrices\n");
       runStep1 = 0;
       lock_residues = 0;
       for(j=0;j<3;j++) {
	 for(k=0;k<3;k++) {
	   /*	   if(argv[i+1+j*3+k][0] != '-') {*/
	     sscanf(argv[i+1+j*3+k],"%f",&(rotation[j][k]));
	     /*	   }*/
	 }
       }
       for(j=0;j<3;j++) {
	 /*	 if(argv[i+9+j][0]!='-') {*/
	   sscanf(argv[i+10+j],"%f",&(translation[j]));
	   /*	 }*/
       }
     }
     else if(strcmp(argv[i], "-sse")==0) {
       runStep1 = 0;
       lock_residues = 1;
     }
     else if (strcmp(argv[i], "-h") == 0) { 
      usageError(); 
    } 
  } 
  
} 
 
 
int checkIfList(FILE *fp) {

  char line[200];
  char word[20];
  int retvalue, done;

  rewind(fp);

  done = 0;
  retvalue = 0;
  while ((fgets(line, 200, fp)) && (done == 0)) {
    if (line[0]!='#') {
      done = 1;
      if(strlen(line)>=9) {
	if(strncmp(line,"LOCK_LIST",9)==0) {
	  retvalue = 1;
	}
      }
    }
  }
  
  rewind(fp);

  return retvalue;
}

int haveSeenAtomBefore(char atoms_seen[MAXLENGTH][5], int num_atoms_seen, char* atom) {
  int i;
  for(i=0;i<num_atoms_seen;i++) {
    if(strcmp(atoms_seen[i],atom)==0) {
      return 1;
    }
  }
  return 0;
}

int haveSeenResidueNumberBefore(OBJ* myobject, int numPts, char* atomnum, char insertionCode) {
  int i;
  int int_atomnum;
  
  sscanf(atomnum,"%d",&int_atomnum);
  for(i=0;i<numPts;i++) {
    if(int_atomnum==myobject->aaNums[i]) {
      if(insertionCode==myobject->insertionCodes[i]) {
	return 1;
      }
    }
  }
  if(myobject->points[numPts][0]!=0 && myobject->points[numPts][1]!=0 && myobject->points[numPts][2]!=0) {
    if(int_atomnum==myobject->aaNums[numPts] && insertionCode==myobject->insertionCodes[i]) {
      return 1;
    }
  }

  return 0;
}

OBJ* readFile(int fnum) { 
 
  OBJ* tempObj;
  int len;
  char syscommand[200];
  int i, imax, count;
  char *path;

  filePtrs[fnum] = fopen(username[fnum], "r");

  if (filePtrs[fnum] != NULL)  {
    if ((fnum == 1) && (globalSearchDatabase == 0)) {
      if (checkIfList(filePtrs[fnum])) {
	globalSearchDatabase = 1;
	globalStartDisplay = 0;
	globalDatabaseFile = filePtrs[fnum];
	return NULL;
      }
    }
    strcpy(filename[fnum], username[fnum]);
    if (fnum == 0) {
      strcpy(pdbname[0], "query");
    }
    else if (fnum == 1) {
      strcpy(pdbname[1], "target");
    }
  

    if ((fnum == 1) && (globalSearchDatabase == 1)) {
      /**********************Amit's method
      imax = strlen(username[1]);
      count = 0;
      
      for(i = 0; i < imax; i++) {
	if (username[1][i] != '/') {	
	  pdbname[1][count] = username[1][i];
	  count++;
	}
	else {
	  count = 0;
	}
      }
      pdbname[1][count] = '\0';
      ******************************/
      path = strrchr(username[1],'/'); /*last occurrence of '/' in username[1]*/
      if(path != NULL) {
	imax = strlen(path);
      
	count = 0;
	for(i=1;i<imax;i++) { /*copy filename beginning from first character after last '/'*/
	  pdbname[1][count] = path[i];
	  count++;
	}
	pdbname[1][count] = '\0';
      }else {
	imax = strlen(username[1]);
	for(i=0;i<imax;i++) {
	  pdbname[1][i] = username[1][i];
	}
	pdbname[1][imax] = '\0';
      }
    }

    newfile[fnum] = 1;
    return objectFromFile(fnum); 
  }    
      
  else {

    len = strlen(username[fnum]);
    
    if (len == 4) {
      strcpy(pdbname[fnum], username[fnum]);   
      if (chain[fnum] != ' ') {
	pdbname[fnum][4] = '-';
	pdbname[fnum][5] = chain[fnum];
	pdbname[fnum][6] = '\0';
      }
    }

    else if ((len == 6) && (username[fnum][4] == '-')) {
      
      strcpy(pdbname[fnum], username[fnum]);

      if (chain[fnum] == ' ') {
	chain[fnum] = pdbname[fnum][5];
      }
      else if (chain[fnum] != pdbname[fnum][5]) {
	pdbname[fnum][5] = chain[fnum];
      }

    }
 
    /*check to see if user entered scop id as a target*/
    else if((len == 7) && (username[fnum][0] == 'd' || username[fnum][0]=='e' || username[fnum][0]=='g')) {
      strcpy(pdbname[fnum], username[fnum]);
      chain[fnum] = ' '; /*chain is encoded in scop id, so leave blank*/
    }
    else {      
      printf("Sorry.  Could not find file %s.\n\n", username[fnum]);
      return NULL;
    }

    newfile[fnum] = 0;
    filename[fnum][0] = '\0';
    sprintf(filename[fnum], "/tmp/%d.file%d", mypid, fnum);

    strcpy(syscommand, scriptdir);
    strcat(syscommand, "/findfile.pl ");
    strcat(syscommand, pdbname[fnum]);
    strcat(syscommand, " ");
    strcat(syscommand, filename[fnum]);
    system(syscommand);

    filePtrs[fnum] = fopen(filename[fnum], "r");

    if (filePtrs[fnum] != NULL) {
      tempObj = objectFromFile(fnum);

      if (filePtrs[fnum] != NULL) {
	strcpy(syscommand, "rm ");
	strcat(syscommand, filename[fnum]);
	system(syscommand);
      }
      return tempObj;
  
    }
    else {      
      printf("Sorry.  Could not find file %s.\n", username[fnum]);    
      return NULL;
    }      
 
  } 
 
}
  

int getSSFromDSSPFile(OBJ* myobject,int fnum) { 

  FILE *infile;
  /*  OBJ *myobject;*/
  int i, j, XX, YY, ZZ;
  char line[200], temp[100], record[10];
  int match, arraylen, atomcount;
  float t0, t1, t2, tempfloat;
  int done = 0;

  infile = filePtrs[fnum];
  rewind(infile); 

  fgets(line, 200, infile);

  while (((line[2] != '#') || (line[13] != 'A') || (line[14] != 'A') || (line[26] != 'B') || (line[27] != 'P') || (line[28] != '1')) && (done == 0)){

    extStr(record, line, 1, 6); 
 

    if (!(fgets(line, 200, infile))) 
      done = 1;
      
  }

  XX = -1;
  if (line[118] == 'X') {
    XX = 116;
    YY = 123;
    ZZ = 130;
  }
  else if (line[110] == 'X') {
    XX = 108;
    YY = 115;
    ZZ = 122;
  }
  else {
    for (i = 0; i < 190; i++) {
      if ((line[i] == 'X') && (line[i+1] == '-') && (line[i+2] == 'C') && (line[i+3] == 'A')) {
	XX = i-2;
	YY = XX + 7;
	ZZ = YY + 7;
	break;
      }
    }
  }
  
  if ((XX == -1) || (done == 1)) {
    printf("\nSorry.  Could not create DSSP file for file %s.\n\n",username[fnum]);
    fclose(infile);
    return -1;
  }else {
    atomcount = 0;
    while (fgets(line, 200, infile)) {
      if ((chain[fnum] == ' ') || (line[11] == chain[fnum])) {
	    /*      if (((chain[fnum] == ' ') || (line[11] == chain[fnum])) && (line[13] != '!') && (line[13]!='X')) {*/
	    if(line[13]!='!' && line[13]!='X') {
	      if (line[16] == 'H') {
		myobject->sstype[atomcount] = 'H';
	      }
	      else if  (line[16] == 'G') {
		myobject->sstype[atomcount] = 'G';
	      }
	      else if (line[16] == 'I') { 
		myobject->sstype[atomcount] = 'I';
	      }
	      else if (line[16] == 'E') {
		myobject->sstype[atomcount] = 'S';
	      }
	      else {
		myobject->sstype[atomcount] = 'U';
	      }
	    }else {
	      myobject->sstype[atomcount] = 'U';
	    }
	  
	    atomcount++;
	  }
    }

    if (atomcount == 0) {
      printf("\nSorry. No CA atoms with chain identifier \"%c\" found in file %s\n\n", chain[fnum], username[fnum]);
      fclose(infile);
      return(-1);
    }else {
      assignSSArray(myobject);
      fclose(infile);
      return(0);
    
    } 
  }
}

void assignSSArray(OBJ* myobject) {
    int arraylen = 0;
    int match = 1;
    int atomcount = myobject->numPts;
    int i,j;

    for (i = 1; i < atomcount; i++) {

	if ((myobject->sstype[i] == myobject->sstype[i-1]) && (myobject->sstype[i] != 'U')) {
	  match++;
	}
	else {
	  if ((myobject->sstype[i-1] == 'H') && (match >= 4)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;
	    arraylen++;
	  }
	  else if ((myobject->sstype[i-1] == 'G') && (match >= 4)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;

	    for (j = i-match; j < i; j++) {
	      myobject->sstype[j] = 'H';
	    }
	    arraylen++;
	  }
	  else if ((myobject->sstype[i-1] == 'I') && (match >= 4)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;

	    for (j = i-match; j < i; j++) {
	      myobject->sstype[j] = 'H';
	    }
	    arraylen++;
	  }
	  else if ((myobject->sstype[i-1] == 'S') && (match >= 3)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;
	    arraylen++;
	  }
	  else {
	    for (j = i-match; j < i; j++) {
	      myobject->sstype[j] = 'U';
	    }
	  }
	  match = 1;
	}
      }
      myobject->ssArrayLength = arraylen;
      myobject->num_SSEs_aligned = 0;
      myobject->num_repeated_SSE_alignments = 0;
      printf("arraylength: %i\n",myobject->ssArrayLength);
      myobject->sstype[myobject->numPts] = '\0';
      /*******
      myobject->ss_alignment = (int**)malloc(sizeof(int*)*(myobject->ssArrayLength+1));
      for(i=0;i<=myobject->ssArrayLength;i++) {
	myobject->ss_alignment[i] = (int*)malloc(sizeof(int)*(myobject->ssArrayLength+1));
	for(j=0;j<myobject->ssArrayLength+1;j++) {
	  myobject->ss_alignment[i][j] = -1;
	}
      }
      ********/
      myobject->num_aligned = (int *)malloc(sizeof(int)*(myobject->ssArrayLength+1));
      myobject->ss_dpscores = (float *)malloc(sizeof(float)*(myobject->ssArrayLength+1));
      for(i=0;i<myobject->ssArrayLength;i++) {
	myobject->num_aligned[i] = 0;
	myobject->ss_dpscores[i] = 0;
      }
      myobject->num_SSEs_aligned = 0;
      myobject->num_repeated_SSE_alignments = 0;
}

void extStr(char* out, char* in, int st, int end) { 
 
  int outcount = 0; 
  int i;

  for (i = st - 1; i < end; i++) { 
 
    out[outcount] = in[i]; 
    outcount++; 
  } 
 
  out[outcount] = '\0'; 
 
} 
 
 
OBJ* objectFromPDBFile(int fnum) { 
 
  FILE* infile; 
  OBJ *myobject, *tempObj; 
  char record[10]; 
  char line[200]; 
  char temp[100]; 
  char atom[10];
  int i, j, k, readingDone = 0, arraylen;
  int atomCount = 0; 
  int CA_atomCount = 0;
  int resChains[MAXLENGTH]; 
  int helices[MAXSS][2]; 
  int sheets[MAXSS][2]; 
  int helixChains[MAXSS]; 
  int sheetChains[MAXSS]; 
  int numhelices = 0; 
  int numsheets = 0; 
  float t0 = 0.0, t1 = 0.0, t2 = 0.0, tempfloat;
  int count, gotss = 0;
  int gotCompound = 0;
  int begin,end,sheet_begin,sheet_end;
  int sheet_length = 4;
  int helix_length = 4;
  int found_beta_carbons = 0;
  int found_beta_carbon = 0;
  int position,found;
  char atomnum[5];
  char old_atomnum[5]="";
  int insertion_code;
  int old_insertion_code;
  int status;
  int found_alpha_carbon=0;
  char atoms_seen[MAXLENGTH][5];
  int num_atoms_seen = 0;
  int incremented_atomCount = 0;

  atomCount=-1;
  infile = filePtrs[fnum]; 
 
  myobject = (OBJ*) malloc(sizeof(OBJ)); 
 
  rewind(infile); 
 
  myobject->compound[0] = '\0'; 
  myobject->objSeq = (char *) malloc((sizeof(char))*(MAXLENGTH));
  myobject->aaNumbers = (int *)malloc((sizeof(int))*MAXLENGTH);
  myobject->header[0] = '\0';
 
  while ((fgets(line, 200, infile)) && (readingDone == 0)) { 

    extStr(record, line, 1, 6); 
 

    if(lock_residues) {
      if(strcmp(record, "REMARK") == 0) {
	extStr(temp,line,11,16);
	sscanf(temp, "%i", &(myobject->num_lock_positions));       /*get the number of residue entries*/
	position = 17;
	for(i=0;i<myobject->num_lock_positions;i++) { /*extract each residue position*/
	  extStr(temp,line,position,position+4);
	  sscanf(temp,"%i",&(myobject->lock_positions[i])); /*this is the residue number, not the index into myobject->points; will be converted at end of function*/
	  position += 5;
	}
      }
    }

    if (strcmp(record, "HEADER") == 0) { 
 
      extStr(myobject->header, line, 11, 50); 
 
    } 
 
    else if ((strcmp(record, "COMPND") == 0) && (gotCompound == 0)) { 
 
      extStr(myobject->compound, line, 11, 70);  
      gotCompound = 1;
      
    } 
 
    else if (strcmp(record, "HELIX ") == 0) { 

 
      if ((chain[fnum] == ' ') || ((line[19] == chain[fnum]) && (line[31] == chain[fnum]))) { 
	/*	printf("line matches chain: %s,%s\n",chain[fnum],line[19]);*/
	gotss = 1;
	extStr(temp, line, 22, 25); 
	sscanf(temp, "%d", &(helices[numhelices][0])); 
	/*	sscanf(temp,"%d",&begin);*/
	extStr(temp, line, 34, 37); 
	sscanf(temp, "%d", &(helices[numhelices][1])); 
	/*	sscanf(temp,"%d",&begin);*/
	/*********
	if((end-begin)>(helix_length-1)) { 
	  printf("breaking up helix %i-%i:\n",begin,end);
	  
	  num = (end-begin+1)/helix_length;
	  remain = num%helix_length;
	  remain2 = remain%num; 
	  helix_begin = begin;
	  helix_end = begin+helix_length-1;
	  if(helix_end > end) {
	    helix_end = end;
	    }else if(helix_end+helix_length > end) {
	      helix_end = end;
	    }
	  while(helix_end < end) {
	    printf("\t%i: %i to %i\n",numhelixs,helix_begin,helix_end);
	    helixs[numhelixs][0] = helix_begin;
	    helixs[numhelixs][1] = helix_end;
	    helix_begin=helix_end+1;
	    helix_end = helix_begin+helix_length-1;
	    if(helix_end > end) {
	      helix_end = end;
	    }else if(helix_end+helix_length > end) {
	      helix_end = end;
	    }
	    helixChains[numhelixs] = line[21];
	    numhelixs++;
	  }
	}
	printf("\t%i: %i to %i\n",numhelices,helix_begin,helix_end);

	helixs[numhelices][0] = helix_begin;
	helixs[numhelices][1] = helix_end;
	helixChains[numhelices] = line[21];
	numhelices++;
	********/
	
	
	helixChains[numhelices] = line[19];
	
	numhelices++; 
      }  
 
    } 
 
    else if (strcmp(record, "SHEET ") == 0) { 
  
      if ((chain[fnum] == ' ') || ((line[21] == chain[fnum]) && (line[32] == chain[fnum]))) { 
 
	gotss = 1;

	extStr(temp, line, 23, 26);
	sscanf(temp, "%d", &(sheets[numsheets][0])); 
	/*	sscanf(temp, "%d",&begin);*/
 
	extStr(temp, line, 34, 37);
		sscanf(temp, "%d", &(sheets[numsheets][1])); 
		/*	sscanf(temp,"%d",&end);*/

		/********************
		  if((end-begin)>(sheet_length-1)) {
	  printf("breaking up sheet %i-%i:\n",begin,end);
	  sheet_begin = begin;
	  sheet_end = begin+sheet_length-1;
	  if(sheet_end > end) {
	    sheet_end = end;
	  }else if(sheet_end+sheet_length > end) {
	    sheet_end = end;
	  }
	  while(sheet_end < end) {
	    printf("\t%i: %i to %i\n",numsheets,sheet_begin,sheet_end);
	    sheets[numsheets][0] = sheet_begin;
	    sheets[numsheets][1] = sheet_end;
	    sheet_begin=sheet_end+1;
	    sheet_end = sheet_begin+sheet_length-1;
	    if(sheet_end > end) {
	      sheet_end = end;
	    }else if(sheet_end+sheet_length > end) {
	      sheet_end = end;
	    }
	    sheetChains[numsheets] = line[21];
	    numsheets++;
	  }
	}else {
	  sheet_begin = begin;
	  sheet_end = end;
	}
	printf("\t%i: %i to %i\n",numsheets,sheet_begin,sheet_end);
	sheets[numsheets][0] = sheet_begin;
	sheets[numsheets][1] = sheet_end;
	sheetChains[numsheets] = line[21];
	numsheets++;
	**********/
      sheetChains[numsheets] = line[21];
	
	numsheets++; 
     

    }  
    
  } 
 
    else if (strcmp(record, "MODEL ") == 0) { 
      printf("NOTE:\tFound multiple models in the PDB file %s. \n\tWill use first model.\n",  username[fnum]); 
    } 
    
    else if (strcmp(record, "ATOM  ") == 0 || strcmp(record,"HETATM")==0) { 
      if((chain[fnum] == ' ') || line[21] == chain[fnum]) {
	extStr(temp,line,23,27);
	sscanf(temp,"%s",&atomnum);
	if(strcmp(atomnum,old_atomnum)!=0) {
	  if(strcmp(old_atomnum,"")==0 || found_alpha_carbon) {
	    if(found_alpha_carbon && found_beta_carbon==0) {
	      findGlycineBetaCarbon(atomCount,myobject,myobject->cbpoints[atomCount]);
	    }
	    atomCount++;
	    incremented_atomCount = 1;
	    num_atoms_seen = 0;
	    found_beta_carbon = 0;
	  }else {
	    /*	    if(incremented_atomCount && atomCount>0) {
	      atomCount--;
	      }*/
	    num_atoms_seen = 0;
	    found_beta_carbon = 0;
	    incremented_atomCount = 0;
	  }
	  found_alpha_carbon = 0;
	}

	
	strcpy(old_atomnum,atomnum);
      }



	extStr(temp, line, 13, 16); 
	sscanf(temp, "%s", atom); 
	if(haveSeenAtomBefore(atoms_seen,num_atoms_seen,atom)) {
	  continue;
	}
	strcpy(atoms_seen[num_atoms_seen],atom);
	num_atoms_seen += 1;
	if(atom[0] == 'N' && strlen(atom) == 1) {
	  if((chain[fnum] == ' ') || line[21] == chain[fnum]) {
	    if(line[21]!=' ') {
	      myobject->chains[atomCount] = line[21];
	    }else{
	      myobject->chains[atomCount]='-';
	    }
	    extStr(temp,line,22,26);
	    sscanf(temp,"%d",&(myobject->aaNums[atomCount]));
	    extStr(temp, line, 18, 20); 
	    myobject->objSeq[atomCount] = convert3to1Code(temp); 
	    myobject->aaNumbers[atomCount] = aaToNumber(myobject->objSeq[atomCount]);
	    myobject->insertionCodes[atomCount]=line[26];
	    extStr(temp, line, 31, 38); 
	    sscanf(temp, "%f", &(N[atomCount][0])); 
	    
	    extStr(temp, line, 39, 46); 
	    sscanf(temp, "%f", &(N[atomCount][1])); 
	    
	    extStr(temp, line, 47, 54); 
	    sscanf(temp, "%f", &(N[atomCount][2])); 

	    resChains[atomCount] = line[21];

	  }
	}else if((atom[0] == 'C') && (strlen(atom) == 1)) {
	  if((chain[fnum] == ' ') || (line[21] == chain[fnum])) {

	    extStr(temp, line, 31, 38); 
	    sscanf(temp, "%f", &(C[atomCount][0])); 
	    
	    extStr(temp, line, 39, 46); 
	    sscanf(temp, "%f", &(C[atomCount][1])); 
	    
	    extStr(temp, line, 47, 54); 
	    sscanf(temp, "%f", &(C[atomCount][2])); 

	    if(myobject->objSeq[atomCount] == 'G') {
	      findGlycineBetaCarbon(atomCount,myobject,myobject->cbpoints[atomCount]);
	      found_beta_carbons = 1;
	    }
	    /*	    atomCount++; */
	  }

      
	}else if ((atom[0] == 'C') && (atom[1] == 'A') && (strlen(atom) == 2)) { 
	  if ((chain[fnum] == ' ') || (line[21] == chain[fnum])) { 
	    found_alpha_carbon = 1;
	    /*	    if(!(myobject->aaNums[atomCount] == myobject->aaNums[atomCount-1])) {*/

	      extStr(temp, line, 23, 26); 
	     sscanf(temp, "%d", &(myobject->aaNums[CA_atomCount])); 


	      extStr(temp, line, 18, 20); 
	      myobject->objSeq[CA_atomCount] = convert3to1Code(temp); 
	      myobject->aaNumbers[CA_atomCount] = aaToNumber(myobject->objSeq[CA_atomCount]);
	      myobject->insertionCodes[CA_atomCount]=line[26];
	      resChains[CA_atomCount] = line[21];

	      extStr(temp, line, 31, 38); 
	      sscanf(temp, "%f", &(myobject->points[CA_atomCount][0])); 
	      
	      extStr(temp, line, 39, 46); 
	      sscanf(temp, "%f", &(myobject->points[CA_atomCount][1])); 
	      
	      extStr(temp, line, 47, 54); 
	      sscanf(temp, "%f", &(myobject->points[CA_atomCount][2])); 

	      CA_atomCount++;
	    
	      /*	      if(myobject->objSeq[atomCount] == 'G') {
		findGlycineBetaCarbon(atomCount,myobject,myobject->cbpoints[atomCount]);
	      }*/
		/*		myobject->cbpoints[atomCount][0] = myobject->points[atomCount][0];
		myobject->cbpoints[atomCount][1] = myobject->points[atomCount][1];
		myobject->cbpoints[atomCount][2] = myobject->points[atomCount][2];
		}*/
	    
	      /****This is now done when nitrogen is read in
	      resChains[atomCount] = line[21];
	      
	      atomCount++; 
	      *****************/
	      /*}*/

	  } 	  
	}else 	if ((atom[0] == 'C') && (atom[1] == 'B') && (strlen(atom) == 2)) { 
	  found_beta_carbons = 1;
	  found_beta_carbon = 1;
	  if ((chain[fnum] == ' ') || (line[21] == chain[fnum])) { 
	      
	      extStr(temp, line, 31, 38); 
	      /*	      sscanf(temp, "%f", &(myobject->cbpoints[atomCount-1][0])); */
	      sscanf(temp, "%f", &(myobject->cbpoints[atomCount][0])); 
	      
	      extStr(temp, line, 39, 46); 
	      /*	      sscanf(temp, "%f", &(myobject->cbpoints[atomCount-1][1])); */
	      sscanf(temp, "%f", &(myobject->cbpoints[atomCount][1])); 
	      
	      extStr(temp, line, 47, 54); 
	      /*	      sscanf(temp, "%f", &(myobject->cbpoints[atomCount-1][2])); */
	      sscanf(temp, "%f", &(myobject->cbpoints[atomCount][2])); 
	      myobject->insertionCodes[atomCount]=line[26];
	  } 	  
	  
	}
      
    }
   
    else if (strcmp(record, "TER   ") == 0) { 
      /*      if ((chain[fnum] != ' ') && (chain[fnum] == resChains[atomCount-1]))*/
      if ((chain[fnum] != ' ') && (chain[fnum] == resChains[atomCount])) 
	readingDone = 1; 
    } 
  
    else if (strcmp(record, "ENDMDL") == 0) { 
      readingDone = 1; 
    } 
 
    else if (strcmp(record, "END   ") == 0) { 
      readingDone = 1; 
    } 
 
  }
 
  if (atomCount == 0 && CA_atomCount==0) { 
 
    printf("\nSorry. No CA atoms with chain identifier \"%c\" found in file %s\n\n", chain[fnum], username[fnum]);
    return NULL;
    
  }

   
  else { 
    atomCount++; 
    printf("atomcount=%i, CA_atomcount=%i\n",atomCount,CA_atomCount);
    if(found_beta_carbons==0) {
      /********      setUseBetaCarbons(0);*/
      useBetaCarbons = 0;
      atomCount = CA_atomCount;
    }
    myobject->numPts = atomCount; 
    myobject->ssArray = (int *)malloc((sizeof(int))*(MAXLENGTH)); 
    myobject->sstype = (char *) malloc((sizeof(char))*(myobject->numPts + 10)); 
    myobject->ssNumbers = (int *)malloc((sizeof(int))*(MAXLENGTH));
    myobject->num_SSEs_aligned = 0;
    myobject->num_repeated_SSE_alignments = 0;

      
    if (gotss == 0) {
      status = noSSinPDBFile(myobject,fnum); 
      fclose(infile);
      myobject->objSeq[atomCount] = '\0';  
      if(status==-1) {
	printf("Sorry.  No secondary structure information in pdb file or dssp file\n");
	return NULL;
      }
    }else {
      arraylen = 0; 
     
      for (i = 0; i < myobject->numPts; i++) { 
	myobject->sstype[i] = 'U'; 
      } 


      for (i = 0; i < atomCount; i++) { 
 
	count = 0; 
 
	for (j = 0; ((j < numhelices) && (count == 0)); j++) { 
 
	  if ((myobject->aaNums[i] >= helices[j][0]) && (myobject->aaNums[i] <= helices[j][1]) && (resChains[i] == helixChains[j])) { 
 
	    /*	  while ((myobject->aaNums[i+count] <= helices[j][1]) && (i+count < atomCount)) {*/
	    while ((myobject->aaNums[i+count] <= helices[j][1])) {
	      count++; 
	      if(i+count >=atomCount) {
		break;
	      }
	    }
 
	    if (count >= 4) { 
	      for (k = 0; k < count; k++) { 
		myobject->sstype[i+k] = 'H'; 
		myobject->ssNumbers[i+k] = arraylen;

	      } 

	      myobject->ssArray[arraylen*2] = i; 
	      myobject->ssArray[arraylen*2 + 1] = i + count; 
	    
	      arraylen++; 
	    } 
	    i = i + count - 1; 
	  } 
	} 
 
 
	for (j = 0; ((j < numsheets) && (count == 0)); j++) { 
	 
	  if ((myobject->aaNums[i] >= sheets[j][0]) && (myobject->aaNums[i] <= sheets[j][1]) && (resChains[i] == sheetChains[j])){ 
	    /*	  while ((myobject->aaNums[i+count] <= sheets[j][1]) &&
	      (i+count < atomCount)) { */
	    while ((myobject->aaNums[i+count] <= sheets[j][1]) ) { 
	      count++; 
	      if(i+count >=atomCount) {
		break;
	      }
	    } 
	    if (count >= 3) { 
	      for (k = 0; k < count; k++) { 
		myobject->sstype[i+k] = 'S'; 
		myobject->ssNumbers[i+k] = arraylen;
	      } 
	      myobject->ssArray[arraylen*2] = i; 
	      myobject->ssArray[arraylen*2 + 1] = i + count; 
	      arraylen++; 
	    } 
	    i = i + count - 1; 
	  } 
	}	       

      }
      myobject->ssArrayLength = arraylen;
      myobject->num_aligned = (int *)malloc(sizeof(int)*(myobject->ssArrayLength+1));
      myobject->ss_dpscores = (float *)malloc(sizeof(float)*(myobject->ssArrayLength+1));
      for(i=0;i<myobject->ssArrayLength;i++) {
	myobject->num_aligned[i] = 0;
	myobject->ss_dpscores[i] = 0;
      }
      
      printf("arraylength: %i\n",myobject->ssArrayLength);
      fclose(infile); 
    }
    for (i = 0; i < myobject->numPts; i++) {
      t0 += myobject->points[i][0];
      t1 += myobject->points[i][1];
      t2 += myobject->points[i][2];
      
      myobject->pairNums[i] = -1;
      
    }
    
    tempfloat = 1.0/((GLfloat)(myobject->numPts));
    
    tempfloat *= -1.0;
    
    t0 *= tempfloat;
    t1 *= tempfloat;
    t2 *= tempfloat;
    
    myobject->objMat[0] = 1.0;
    myobject->objMat[1] = 0.0;
    myobject->objMat[2] = 0.0;
    myobject->objMat[3] = 0.0;
    
    myobject->objMat[4] = 0.0;
    myobject->objMat[5] = 1.0;
    myobject->objMat[6] = 0.0;
    myobject->objMat[7] = 0.0;
    
    myobject->objMat[8] = 0.0;
    myobject->objMat[9] = 0.0;
    myobject->objMat[10] = 1.0;
    myobject->objMat[11] = 0.0;
    
    if(runStep1) {
      myobject->objMat[12] = t0;
      myobject->objMat[13] = t1;
      myobject->objMat[14] = t2;
      myobject->objMat[15] = 1.0;
      myobject->center[0] = 0.0;
      myobject->center[1] = 0.0;
      myobject->center[2] = 0.0;
    }else {
      myobject->objMat[12]=0;myobject->objMat[13]=0;myobject->objMat[14]=0;myobject->objMat[15]=1;
      myobject->center[0] = -t0;
      myobject->center[1] = -t1;
      myobject->center[2] = -t2;
    }
        


#ifdef DISPLAYON
    if (globalStartDisplay) {
      myobject->modelNumber = glGenLists(1);
      myobject->color[0] = colorTable[fnum][0];
      myobject->color[1] = colorTable[fnum][1];
      myobject->color[2] = colorTable[fnum][2];
            
      createObjList(myobject);
    }
#endif


    /************
    myobject->ss_alignment = (int**)malloc(sizeof(int*)*(myobject->ssArrayLength+1));
    myobject->num_SSEs_aligned = 0;
    for(i=0;i<=myobject->ssArrayLength;i++) {
      myobject->ss_alignment[i] = (int*)malloc(sizeof(int)*(myobject->ssArrayLength+1));
      for(j=0;j<myobject->ssArrayLength;j++) {
	myobject->ss_alignment[i][j] = -1;
      }
    }
    ********/
    if(lock_residues) {
      /*convert myobject->lock_positions from residue numbers to indices into myobject->points*/
      for(i=0;i<myobject->num_lock_positions;i++) {
	found = 0;
	for(j=0;j<myobject->numPts;j++) {
	  if(myobject->aaNums[j] == myobject->lock_positions[i]) {
	    myobject->lock_positions[i] = j;
	    found = 1;
	    break;
	  }
	}
	if(!found) {
	  myobject->lock_positions[i] = -1;
	}
      }
    }

      myobject->sstype[myobject->numPts] = '\0';
      myobject->objSeq[myobject->numPts] = '\0';  

    return myobject;
    

  } 

}

/*findGlycineBetaCarbon: Places a "fake" beta carbon on glycine residues.
 *This is done by rotating the carbonyl carbon about the N-CA bond by 120 degrees.
 */
void findGlycineBetaCarbon(int index, OBJ* myobject, float *vec) {
  int i;
  float CA[4];
  float myN[4];
  float myC[4];
  float a,b,c,d;
  float a2,b2,c2,a3,b3,c3,a4,b4,c4;
  float length;
  double theta;
  float R[4][4]; /*matrix for rotation about x axis*/
  float Ri[4][4]; /*inverse of R*/


  theta =-2.0*PI;
  theta /= ((double)3.0);


  for(i=0;i<3;i++) {
    CA[i] = myobject->points[index][i];
    myN[i] = N[index][i];
    myC[i] = C[index][i];
  }
  CA[3] = 1; myN[3] = 1; myC[3] = 1;

  /*Calculate a unit vector (a,b,c) along the CA-N axis*/
  a = myN[0]-CA[0];
  b = myN[1]-CA[1];
  c = myN[2]-CA[2];
  length = sqrt(a*a+b*b+c*c);
  a /= length; b/=length; c/=length;

  /*Translate atoms such that the alpha carbon is at the origin*/
  for(i=0;i<3;i++) {
    myN[i] -= CA[i];
    myC[i] -= CA[i];
    /*    CA[i] = 0;*/
  }

  /*Rotate space about the x axis so that the rotation axis lies in the xz plane*/
  d = sqrt(b*b+c*c);
  if(d != 0) {
    a2 = myC[0];
    b2 = myC[1]*c/d - myC[2]*b/d;
    c2 = myC[1]*b/d + myC[2]*c/d;
  }else {
    a2 = myC[0];
    b2 = myC[1];
    c2 = myC[2];
  }

  /*Rotate space about the y axis so that the rotation axis lies along the z axis*/
  a3 = a2*d-c2*a;
  b3 = b2;
  c3 = a*a2+d*c2;

  /*Rotation about the z axis by 120 degrees: check sign in a4 equation*/
  a4 = cos(theta)*a3+sin(theta)*b3;
  b4 = -sin(theta)*a3+cos(theta)*b3;
  c4 = c3;

  /*invert rotation about y axis*/
  a3 = d*a4+a*c4;
  b3 = b4;
  c3 = -a*a4+d*c4;

  /*invert rotation about x axis*/
  if(d != 0) {
    a2 = a3;
    b2 = b3*c/d + c3*b/d;
    c2 = -b3*b/d + c3*c/d;
  }else {
    a2 = a3; b2 = b3; c2 = c3;
  }
  
  /*invert translation to get final point*/
  vec[0] = a2+CA[0];
  vec[1] = b2+CA[1];
  vec[2] = c2+CA[2];
  /*check the distance
  d=0;
  for(i=0;i<3;i++) {
    d += (vec[i]-myobject->points[index][i])*(vec[i]-myobject->points[index][i]);
  }
  d = sqrt(d);
  */
}

 
void getCBAtoms(OBJ* myobject, FILE *pdbfile, int fnum) {
  int atomCount = 0, readingDone = 0;
  char record[10];
  char line[200];
  char atom[10];
  char temp[100];
  int resChains[MAXLENGTH];
  char old_atomnum[5]="";
  char atomnum[5];
 
  rewind(pdbfile);
  atomCount=-1;
  while ((fgets(line, 200, pdbfile)) && (readingDone == 0)) { 

    extStr(record, line, 1, 6); 
    
    if (strcmp(record, "ATOM  ") == 0 || strcmp(record, "HETATM")==0) { 
	if((chain[fnum] == ' ') || line[21] == chain[fnum]) {
	  extStr(temp,line,23,27);
	  sscanf(temp,"%s",&atomnum);
	  if(strcmp(atomnum,old_atomnum)!=0) {
	    atomCount++;
	  }
	}
      strcpy(old_atomnum,atomnum);
      extStr(temp, line, 13, 16); 
      sscanf(temp, "%s", atom);
      if(atom[0] == 'N' && strlen(atom) == 1) {

	if((chain[fnum] == ' ') || line[21] == chain[fnum]) {


	    extStr(temp,line,23,26);
	    /*    sscanf(temp,"%d",&(myobject->aaNums[atomCount]));*/
	    extStr(temp, line, 18, 20); 
	    /*	    myobject->objSeq[atomCount] = convert3to1Code(temp); 
		    myobject->aaNumbers[atomCount] = aaToNumber(myobject->objSeq[atomCount]);	      */
	    extStr(temp, line, 31, 38); 
	    sscanf(temp, "%f", &(N[atomCount][0])); 
	    
	    extStr(temp, line, 39, 46); 
	    sscanf(temp, "%f", &(N[atomCount][1])); 
	    
	    extStr(temp, line, 47, 54); 
	    sscanf(temp, "%f", &(N[atomCount][2])); 

	    resChains[atomCount] = line[21];
	}
      }else if((atom[0] == 'C') && (strlen(atom) == 1)) {
	  if((chain[fnum] == ' ') || (line[21] == chain[fnum])) {
	    extStr(temp, line, 31, 38); 
	    sscanf(temp, "%f", &(C[atomCount][0])); 
	    
	    extStr(temp, line, 39, 46); 
	    sscanf(temp, "%f", &(C[atomCount][1])); 
	    
	    extStr(temp, line, 47, 54); 
	    sscanf(temp, "%f", &(C[atomCount][2])); 

	    if(myobject->objSeq[atomCount] == 'G') {
	      findGlycineBetaCarbon(atomCount,myobject,myobject->cbpoints[atomCount]);

	    }
	    /*	      atomCount++;*/

	  }
      }

      extStr(temp, line, 18, 20);
      if(strcmp(temp, "GLY") == 0 && ((atom[0] == 'C') && (atom[1] == 'A'))) {
	/*myobject->cbpoints[atomCount][0] = myobject->points[atomCount][0];
	myobject->cbpoints[atomCount][1] = myobject->points[atomCount][1];
	myobject->cbpoints[atomCount][2] = myobject->points[atomCount][2];
	atomCount++;*/
      }else if ((atom[0] == 'C') && (atom[1] == 'B') && (strlen(atom) == 2)) { 
	if ((chain[fnum] == ' ') || (line[21] == chain[fnum])) { 
	  
	  /*	  resChains[atomCount-1] = line[21];*/
	  resChains[atomCount] = line[21];
	  extStr(temp, line, 18, 20); 
	  extStr(temp, line, 31, 38); 
	  /*	  sscanf(temp, "%f", &(myobject->cbpoints[atomCount-1][0])); */
	  sscanf(temp, "%f", &(myobject->cbpoints[atomCount][0])); 
	  
	  extStr(temp, line, 39, 46); 
	  /*	  sscanf(temp, "%f", &(myobject->cbpoints[atomCount-1][1])); */
	  sscanf(temp, "%f", &(myobject->cbpoints[atomCount][1])); 
	    
	  extStr(temp, line, 47, 54); 
	  /*	  sscanf(temp, "%f", &(myobject->cbpoints[atomCount-1][2])); */
	  sscanf(temp, "%f", &(myobject->cbpoints[atomCount][2])); 
	  /*atomCount++;*/
	    
	}
      }
	
    }else if (strcmp(record, "TER   ") == 0) { 
      /*      if ((chain[fnum] != ' ') && (chain[fnum] == resChains[atomCount-1]))*/
      if ((chain[fnum] != ' ') && (chain[fnum] == resChains[atomCount]))
	readingDone = 1; 
    } 
  
    else if (strcmp(record, "ENDMDL") == 0) { 
      readingDone = 1; 
    } 
 
    else if (strcmp(record, "END   ") == 0) { 
      readingDone = 1; 
    } 
      
  }
}



 
int noSSinPDBFile(OBJ* myobject, int fnum) { 
  int status;
  char syscommand[200]; 
  char line[200];
  OBJ* tempObj;
  char tempname[200];
  
  if (newfile[fnum] == 1) {
    strcpy(tempname, filename[fnum]);

    printf("No secondary structure information in file %s\n", username[fnum]);
    if(strcmp(dsspname[fnum], "NONE") != 0) {
      printf("Will obtain secondary structure information from the dssp file\n");
      strcpy(filename[fnum],dsspname[fnum]);
    }else {
      printf("Will now try to create a DSSP file for %s\n\n", username[fnum]);
      
      filename[fnum][0] = '\0';
      sprintf(filename[fnum], "/tmp/%d.dssp%d", mypid, fnum);
      
      strcpy(syscommand, dsspprogram);
      strcat(syscommand, " ");
      strcat(syscommand, tempname);
      strcat(syscommand, " ");
      strcat(syscommand, filename[fnum]);
      printf("running command %s\n",syscommand);
      system(syscommand);
      
    }
      filePtrs[fnum] = NULL;
      
      filePtrs[fnum] = fopen(filename[fnum], "r");
      
    if (filePtrs[fnum] != NULL) {
      
      fgets(line, 200, filePtrs[fnum]); 
      
      if ((line[51] == 'D') && (line[52] == 'S') && (line[53] == 'S') && (line[54] == 'P')) { 
	status = getSSFromDSSPFile(myobject,fnum);

	if(strcmp(dsspname[fnum],"NONE") == 0) {
	  strcpy(syscommand, "rm ");
	  strcat(syscommand, filename[fnum]);
	  system(syscommand); 
	}
	
	return status;

      } 
      
      else { 
	printf("Sorry.  Could not create a DSSP file for this protein.\n"); 
	return -1; 
      } 
    }      
    else { 
      printf("Sorry.  Could not create a DSSP file for this protein.\n"); 
      return -1; 
    } 
  }
  else {
    printf("No secondary structure information in file %s\n", pdbname[fnum]);
    if(strcmp(dsspname[fnum], "NONE") != 0) {
      printf("Will obtain secondary structure information from the dssp file\n");
      strcpy(filename[fnum],dsspname[fnum]);
    }else {
      printf("Will now try to find a DSSP file for %s.\n\n", pdbname[fnum]); 
   
      strcpy(syscommand, "rm ");
      strcat(syscommand, filename[fnum]);
      system(syscommand);
      
      filename[fnum][0] = '\0';
      sprintf(filename[fnum], "/tmp/%d.dssp%d", mypid, fnum);
    
      strcpy(syscommand, scriptdir);
      strcat(syscommand, "/findDSSPfile.pl ");
      strcat(syscommand, pdbname[fnum]);
      strcat(syscommand, " ");
      strcat(syscommand, filename[fnum]);
      printf("dssp command: %s\n",syscommand);
      system(syscommand);
    }
    filePtrs[fnum] = NULL;

    printf("filename: %s\n",filename[fnum]);
    filePtrs[fnum] = fopen(filename[fnum], "r");

    if (filePtrs[fnum] != NULL) {
      
      fgets(line, 200, filePtrs[fnum]); 
      
      if ((line[51] == 'D') && (line[52] == 'S') && (line[53] == 'S') && (line[54] == 'P')) { 
	status = getSSFromDSSPFile(myobject,fnum); 
      } 
      
      else { 
	printf("Sorry.  Could not find a DSSP file for this protein.\n"); 
	return status; 
      } 
    }      
    else { 
      printf("Sorry.  Could not find a DSSP file for this protein.\n"); 
      return status; 
    } 
  } 
    
} 
 
 
 
 
OBJ* objectFromFile(int fnum) { 
 
  char line[200]; 
  char record[10];

  rewind(filePtrs[fnum]);

  fgets(line, 200, filePtrs[fnum]); 
   
  /*  if ((line[51] == 'D') && (line[52] == 'S') && (line[53] == 'S') && (line[54] == 'P')) { 
    return objectFromDSSPFile(fnum); 
 
    } */
   
  if ((line[0] == 'H') && (line[1] == 'E') && (line[2] == 'A') && (line[3] == 'D') && (line[4] == 'E') && (line[5] == 'R')) { 
    return objectFromPDBFile(fnum); 
 
  } 
  else { 
    while(fgets(line,200,filePtrs[fnum])) {
      extStr(record,line,1,6);
      if(strcmp(record,"ATOM  ") == 0 || strcmp(record, "HETATM")==0) {
	return objectFromPDBFile(fnum);
      }
    }
    fclose(filePtrs[fnum]);
    printf("Sorry.  The input file does not appear to be in either PDB or DSSP format\n"); 
    return NULL;
  }  
} 
 

void assignSSENumbers(OBJ* protein) {
  int i;
  int new_SSE = 0;
  int SSE_count = 0;
  char prev_ss_type = ' ';
  protein->ssNumbers = (int *)malloc(sizeof(int)*protein->numPts);
  
  prev_ss_type = protein->sstype[0]; /*initialization*/
  protein->ssNumbers[0] = SSE_count;

  for(i=0;i<protein->numPts;i++) {
   if(protein->sstype[i] == prev_ss_type) {
      protein->ssNumbers[i] = SSE_count;
    }else if(protein->sstype[i] != 'U') {
      protein->ssNumbers[i]=++SSE_count;
      prev_ss_type = protein->sstype[i];
    }else{
      protein->ssNumbers[i] = -1;
    }
   printf("%i\n",protein->ssNumbers[i]);
  }

}
